use crate::*;

/*
 * Function to assemble anti-dependence edges. Returns a list of pairs of nodes.
 * The first item in the pair is the read node, and the second item is the write
 * node.
 */
pub fn antideps(function: &Function, def_use: &ImmutableDefUseMap) -> Vec<(NodeID, NodeID)> {
    // Anti-dependence edges are between a write node and a read node, where
    // each node uses the same array value. The read must be scheduled before
    // the write to avoid incorrect compilation.
    let mut antideps = vec![];

    for id in (0..function.nodes.len()).map(NodeID::new) {
        // Collect the reads and writes to / from this collection.
        let users = def_use.get_users(id);
        let reads = users.iter().filter(|user| {
            if let Node::Read {
                collect,
                indices: _,
            } = function.nodes[user.idx()]
            {
                collect == id
            } else {
                false
            }
        });
        let mut writes = users.iter().filter(|user| {
            if let Node::Write {
                collect,
                data: _,
                indices: _,
            } = function.nodes[user.idx()]
            {
                collect == id
            } else {
                false
            }
        });

        // If there are any writes, compute the anti dependence edges.
        if let Some(write) = writes.next() {
            for read in reads {
                antideps.push((*read, *write));
            }
        }
    }

    antideps
}
