extern crate clap;
extern crate postcard;

use std::fs::File;
use std::io::prelude::*;
use std::path::Path;

use clap::Parser;

#[derive(Parser, Debug)]
#[command(author, version, about, long_about = None)]
struct Args {
    file: String,
    passes: String,
}

fn parse_file_from_hir(path: &Path) -> hercules_ir::ir::Module {
    let mut file = File::open(path).expect("PANIC: Unable to open input file.");
    let mut contents = String::new();
    file.read_to_string(&mut contents)
        .expect("PANIC: Unable to read input file contents.");
    hercules_ir::parse::parse(&contents).expect("PANIC: Failed to parse Hercules IR file.")
}

fn parse_file_from_hbin(path: &Path) -> hercules_ir::ir::Module {
    let mut file = File::open(path).expect("PANIC: Unable to open input file.");
    let mut buffer = vec![];
    file.read_to_end(&mut buffer).unwrap();
    postcard::from_bytes(&buffer).unwrap()
}

fn main() {
    let args = Args::parse();
    assert!(
        args.file.ends_with(".hir") || args.file.ends_with(".hbin"),
        "PANIC: Running hercules_driver on a file without a .hir or .hbin extension."
    );
    let path = Path::new(&args.file);
    let module = if args.file.ends_with(".hir") {
        parse_file_from_hir(path)
    } else {
        parse_file_from_hbin(path)
    };

    let mut pm = hercules_opt::pass::PassManager::new(module);
    let passes: Vec<hercules_opt::pass::Pass> = args
        .passes
        .split(char::is_whitespace)
        .map(|pass_str| {
            assert_ne!(
                pass_str, "",
                "PANIC: Can't interpret empty pass name. Try giving a list of pass names."
            );
            ron::from_str(pass_str).expect("PANIC: Couldn't parse list of passes.")
        })
        .collect();
    for pass in passes {
        pm.add_pass(pass);
    }
    pm.run_passes();
}
