extern crate clap;

use juno_compiler::*;

use clap::{ArgGroup, Parser};

use std::path::PathBuf;

#[derive(Parser)]
#[clap(author, version, about, long_about = None)]
#[clap(group(
        ArgGroup::new("scheduling")
            .required(false)
            .args(&["schedule", "default_schedule", "no_schedule"])))]
struct Cli {
    src_file: String,
    #[clap(short, long)]
    verify: bool,
    #[clap(long = "verify-all")]
    verify_all: bool,
    #[arg(short, long = "x-dot")]
    x_dot: bool,
    #[clap(short, long, value_name = "SCHEDULE")]
    schedule: Option<String>,
    #[clap(short, long = "default-schedule")]
    default_schedule: bool,
    #[clap(short, long)]
    no_schedule: bool,
    #[arg(short, long = "output-dir", value_name = "OUTPUT DIR")]
    output_dir: Option<String>,
}

fn main() {
    let args = Cli::parse();
    let verify = if args.verify_all {
        JunoVerify::AllPasses
    } else if args.verify {
        JunoVerify::JunoOpts
    } else {
        JunoVerify::None
    };
    let schedule = match args.schedule {
        Some(file) => JunoSchedule::Schedule(file),
        None => {
            if args.default_schedule {
                JunoSchedule::DefaultSchedule
            } else {
                JunoSchedule::None
            }
        }
    };
    let output_dir = match args.output_dir {
        Some(dir) => dir,
        None => PathBuf::from(args.src_file.clone())
            .parent()
            .unwrap()
            .to_str()
            .unwrap()
            .to_string(),
    };
    match compile(args.src_file, verify, args.x_dot, schedule, output_dir) {
        Ok(()) => {}
        Err(errs) => {
            eprintln!("{}", errs);
        }
    }
}
