%start Schedule

%avoid_insert "ID" "LABEL"
%expect-unused Unmatched 'UNMATCHED'

%%

Schedule -> Vec<FuncDirectives> : FunctionList { $1 };

FunctionList -> Vec<FuncDirectives>
  : { vec![] }
  | FunctionList FunctionDef { snoc($1, $2) }
  ;

FunctionDef -> FuncDirectives
  : 'function' Func '{' DirectiveList '}'
    { FuncDirectives { span : $span, func : $2, directives : $4 }};

DirectiveList -> Vec<Directive>
  : { vec![] }
  | DirectiveList Directive { snoc($1, $2) }
  ;

Directive -> Directive
  : 'partition' Labels 'on' Devices
      { Directive::Partition { span : $span, labels : $2, devices : $4 } }
  | 'ID' Labels
      { Directive::Schedule { span : $span, command : span_of_tok($1), args : $2 } }
  ;

Func -> Func
  : 'ID' { Func { span : $span, name : $span, }}
  ;

Labels -> Vec<Span>
  : 'LABEL' { vec![span_of_tok($1)] }
  | '{' LabelsRev '}' { rev($2) }
  ;
LabelsRev -> Vec<Span>
  : { vec![] }
  | 'LABEL' { vec![span_of_tok($1)] }
  | 'LABEL' ',' LabelsRev { cons(span_of_tok($1), $3) }
  ;

Devices -> Vec<Device>
  : Device { vec![$1] }
  | '{' SomeDevices '}' { $2 }
  ;
SomeDevices -> Vec<Device>
  : Device { vec![$1] }
  | SomeDevices ',' Device { snoc($1, $3) }
  ;

Device -> Device
  : 'ID'
      { Device { span : $span, name : span_of_tok($1), } }
  ;

Unmatched -> () : 'UNMATCHED' {};

%%

use cfgrammar::Span;
use lrlex::DefaultLexeme;

fn span_of_tok(t : Result<DefaultLexeme, DefaultLexeme>) -> Span {
  t.map_err(|_| ()).map(|l| l.span()).unwrap()
}

fn cons<A>(hd : A, mut tl : Vec<A>) -> Vec<A> {
  tl.push(hd);
  tl
}

fn snoc<A>(mut hd : Vec<A>, tl : A) -> Vec<A> {
  hd.push(tl);
  hd
}

fn rev<A>(mut lst : Vec<A>) -> Vec<A> {
  lst.reverse();
  lst
}

pub struct Func { pub span : Span, pub name : Span, }
pub struct Device { pub span : Span, pub name : Span, }

pub struct FuncDirectives { pub span : Span, pub func : Func,
                            pub directives : Vec<Directive> }

pub enum Directive {
  Schedule  { span : Span, command : Span, args : Vec<Span> },
  Partition { span : Span, labels : Vec<Span>, devices : Vec<Device> },
}
